/******************************************************************************/
/*                                                                            */
/* Program name: mqtrcstats                                                   */
/*                                                                            */
/* Author: Tim Zielke - MQ Administrator for Aon Hewitt.                      */ 
/*                                                                            */
/* Description: MQ administrative tool to pull statistics from a distributed  */
/*              MQ api trace.                                                 */
/*              RECOMMENDATION: Run trace as follows to get relatively        */
/*                              accurate MQI response times.                  */
/*                                                                            */ 
/*              strmqtrc -m qmgr -p program -t api                            */
/*                                                                            */
/* Function:                                                                  */
/*                                                                            */
/* This program will take a formatted distributed MQ api trace and attempt    */
/* to pull meaningful statistics such as the following:                       */
/*                                                                            */
/* MQOPEN by HCONN with total and average MQI response time                   */
/* MQCLOSE by HCONN with total and average MQI response time                  */
/* MQGET by queue name with total and average MQI response time               */
/* MQPUT by queue name with total and average MQI response time               */
/* MQPUT1 by queue name with total and average MQI response time              */
/*                                                                            */
/* The author of this program does not have proprietary knowledge of the      */
/* format of the distributed MQ api trace, so this is a best effort approach  */
/* that is clearly not exhaustive or bullet-proof.  However, based on         */
/* testing it does fairly accurately pull out and summarize the statistical   */
/* information.                                                               */
/*                                                                            */
/* This program was tested on Linux x86_64 and Solaris 10 SPARC.              */
/*                                                                            */
/* If the program does not find MQI trace entries formatted in a certain      */
/* way, it will print warnings at the beginning of the output.  You can       */
/* ignore this, but take it as MQI trace entries are probably being missed.   */
/*                                                                            */
/* mqtrcstats has the following inputs:                                       */
/*    required:                                                               */
/*              (1) name of formatted trace file                              */
/*    optional:                                                               */
/*              (2) endianness of server that the trace was run on            */
/*                                                                            */
/* NOTE: endianness is needed to properly read multi-byte binary data in the  */
/*       trace. If you are not sure what endianness is, just run mqtrcstats   */
/*       on the same server that you ran that trace on and do not specify the */
/*       2nd input, and the program will try and determine the endianness of  */
/*       the server for you.                                                  */
/******************************************************************************/  
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#define READMAX 4096
#define QSTATMAX 100 
#define QNAMEMAX 48 
#define HCONNMAX 8
#define HEXSTRMAX 8 

double getElapsedTime(char begTime[], char endTime[]);
long ConvertHexToDec(char HexStr[], int len);
long ConvertHexToDecLittleEndianLen8(char HexStr[]); 

int main(int argc, char *argv[])
{

   char *str1, *str2, *str3;
   char lineBuf[READMAX+1], begTime[16], endTime[16], HexStr[HEXSTRMAX+1], QueueName[QNAMEMAX+1], HConn[HCONNMAX+1], Compcode[9];

   lineBuf[READMAX]='\0';
   begTime[15]='\0'; 
   endTime[15]='\0';
   HexStr[HEXSTRMAX]='\0';
   QueueName[QNAMEMAX]='\0';
   HConn[HCONNMAX]='\0';
   Compcode[8]='\0';

   char littleEndian, findCd;
   FILE *fp;
   int i, j;
   long lineNmbr, DataLen, MQOPENbadFormat, MQCLOSEbadFormat, MQGETbadFormat, MQPUTbadFormat, MQPUT1badFormat;
   double elapsedTime, avgOkElapsedTime, avgFailedElapsedTime;
   int getMsgOptsLine, putMsgOptsLine, put1MsgOptsLine, put1BufferlengthLine; 

   struct QueueStats {
      char QueueName[QNAMEMAX];
      char break1;
      char HConn[HCONNMAX];
      char break2;
      long TotOk;
      long TotFailed;
      long TotDataLen;
      double TotOkElapsedTime;
      double TotFailedElapsedTime;
   };

   char MQOPENBegFindCd, MQOPENBegReasonFindCd, MQOPENEndFindCd, MQOPENEndCompFindCd, MQIOPENFindCd, MQOPENQueueStatsFindCd;

   int MQOPENEndCompCtr;

   int MQOPENQueueStatsArCt;
   struct QueueStats MQOPENQueueStats[QSTATMAX];

   char MQCLOSEBegFindCd, MQCLOSEEndFindCd, MQCLOSEEndCompFindCd, MQICLOSEFindCd, MQCLOSEQueueStatsFindCd;

   int MQCLOSEEndCompCtr;

   int MQCLOSEQueueStatsArCt;
   struct QueueStats MQCLOSEQueueStats[QSTATMAX];

   char MQGETBegFindCd, MQGETBegReasonFindCd, MQGETEndFindCd, MQGETEndCompFindCd, MQIGETDataLenFindCd, MQGETMsgOptsFindCd, MQGETQueueStatsFindCd;

   int MQGETEndCompCtr;

   int MQGETQueueStatsArCt;
   struct QueueStats MQGETQueueStats[QSTATMAX];

   char MQPUTBegFindCd, MQPUTBegReasonFindCd, MQPUTEndFindCd, MQPUTEndCompFindCd, MQIPUTDataLenFindCd, MQPUTMsgOptsFindCd, MQPUTQueueStatsFindCd;

   int MQPUTEndCompCtr;

   int MQPUTQueueStatsArCt;
   struct QueueStats MQPUTQueueStats[QSTATMAX];

   char MQPUT1BegFindCd, MQPUT1BegReasonFindCd, MQPUT1EndFindCd, MQPUT1EndCompFindCd, MQIPUT1DataLenFindCd, MQPUT1MsgOptsFindCd, MQPUT1QueueStatsFindCd;

   int MQPUT1EndCompCtr;

   int MQPUT1QueueStatsArCt;
   struct QueueStats MQPUT1QueueStats[QSTATMAX];

   union endianness {
      long s;
      char c[sizeof(long)];
   }  un;

/* 1st argument is required (1 - trace input file), 2nd argument is optional (2 - endianness) */

   if (argc < 2)
      {
       printf("Usage is:  mqtrcstats file endianness\n");
       printf("Where file (required input) is the name of a formatted MQ trace file (ex. AMQ3853.0.FMT).\n");
       printf("Where endianness (optional input) is the endianness of the server that the trace was run on (ex. big or little).\n");
       printf("If you do not specify endianness, the endianness of the server that you are running mqtrcstats on is determined and used for that value.\n");
       printf("ex:  mqtrcstats AMQ3853.0.FMT\n");
       printf("ex:  mqtrcstats AMQ3853.0.FMT little\n");
       exit(-1);
      }   

/* Get formatted trace file from first command line argument */

   if ((fp = fopen(argv[1], "r")) == NULL)
      {
       printf("Unable to open %s, errno was %d, ending program.\n", argv[1], errno);
       exit(-1);
      }

/* Determine endianess */

   if (argc == 3)
      {
       findCd = 'N'; 
       if (strcmp(argv[2],"little") == 0)
          {
           littleEndian = 'Y'; 
           findCd = 'Y';
          }
       if (strcmp(argv[2],"big") == 0)
          {
           littleEndian = 'N';
           findCd = 'Y';
          }       
       if (findCd == 'N')
          {
           printf("2nd input for endianess must be a value of big or little\n");
           exit(-1);
          }  
      }
   else
      {
       un.s = 0x0001;
       if (sizeof(long) > 1) 
          {
           if (un.c[0] == 1)
              {
               littleEndian = 'Y';
               printf("\nmqtrcstats determined current server is little endian. We will assume the trace file was generated on a little endian server.\n\n");
              }
           else
              {
               littleEndian = 'N';
               printf("\nmqtrcstats determined current server is big endian. We will assume the trace file was generated on a big endian server.\n\n");
              }
          }
       else
          {
           printf("Unable to determine endianess. Ending program\n");
           exit(-1);
          }
      } 

/* Initialize MQOPEN variables */

   MQOPENbadFormat=0;
   MQOPENBegFindCd='N';
   MQOPENBegReasonFindCd='N';
   MQOPENEndFindCd='Y';
   MQIOPENFindCd='N';
   MQOPENEndCompFindCd='N';
   MQOPENEndCompCtr=0;
   MQOPENQueueStatsArCt=0;

   for (i = 0; i < QSTATMAX; i++)
      {
       for (j = 0; j < QNAMEMAX; j++)
           MQOPENQueueStats[i].QueueName[j]=' ';
       MQOPENQueueStats[i].break1='\0'; 
       for (j = 0; j < HCONNMAX; j++)
           MQOPENQueueStats[i].HConn[j]=' ';
       MQOPENQueueStats[i].break2='\0';
       MQOPENQueueStats[i].TotOk=0;
       MQOPENQueueStats[i].TotFailed=0;
       MQOPENQueueStats[i].TotDataLen=0;
       MQOPENQueueStats[i].TotOkElapsedTime=0;
       MQOPENQueueStats[i].TotFailedElapsedTime=0;
      }
   j=QSTATMAX-1;
   if (j >= 0)
      strncpy(MQOPENQueueStats[j].QueueName,"OVERFLOW SLOT: QUEUE NAME MAX COUNT EXCEEDED",44);

/* Initialize MQCLOSE variables */

   MQCLOSEbadFormat=0;
   MQCLOSEBegFindCd='N';
   MQCLOSEEndFindCd='Y';
   MQICLOSEFindCd='N';
   MQCLOSEEndCompFindCd='N';
   MQCLOSEEndCompCtr=0;
   MQCLOSEQueueStatsArCt=0;

   for (i = 0; i < QSTATMAX; i++)
      {
       for (j = 0; j < QNAMEMAX; j++)
           MQCLOSEQueueStats[i].QueueName[j]=' ';
       MQCLOSEQueueStats[i].break1='\0';
       for (j = 0; j < HCONNMAX; j++)
           MQCLOSEQueueStats[i].HConn[j]=' ';
       MQCLOSEQueueStats[i].break2='\0';
       MQCLOSEQueueStats[i].TotOk=0;
       MQCLOSEQueueStats[i].TotFailed=0;
       MQCLOSEQueueStats[i].TotDataLen=0;
       MQCLOSEQueueStats[i].TotOkElapsedTime=0;
       MQCLOSEQueueStats[i].TotFailedElapsedTime=0;
      }
   j=QSTATMAX-1;
   if (j >= 0)
      strncpy(MQCLOSEQueueStats[j].HConn,"OVERFLOW",8);

/* Initialize MQGET variables */

   MQGETbadFormat=0;
   MQGETBegFindCd='N';
   MQGETBegReasonFindCd='N';
   MQGETEndFindCd='Y';
   MQGETEndCompFindCd='N';
   MQGETEndCompCtr=0;
   MQIGETDataLenFindCd='N';
   MQGETMsgOptsFindCd='N'; 
   getMsgOptsLine=0;
   MQGETQueueStatsArCt=0;

   for (i = 0; i < QSTATMAX; i++)
      {
       for (j = 0; j < QNAMEMAX; j++)
           MQGETQueueStats[i].QueueName[j]=' ';  
       MQGETQueueStats[i].break1='\0';
       MQGETQueueStats[i].TotOk=0;
       MQGETQueueStats[i].TotFailed=0;
       MQGETQueueStats[i].TotDataLen=0;
       MQGETQueueStats[i].TotOkElapsedTime=0;
       MQGETQueueStats[i].TotFailedElapsedTime=0;     
      }
   j=QSTATMAX-1;
   if (j >= 0)
      strncpy(MQGETQueueStats[j].QueueName,"OVERFLOW SLOT: QUEUE NAME MAX COUNT EXCEEDED",44); 

/* Initialize MQPUT variables */

   MQPUTbadFormat=0;
   MQPUTBegFindCd='N';
   MQPUTBegReasonFindCd='N';
   MQPUTEndFindCd='Y';
   MQPUTEndCompFindCd='N';
   MQPUTEndCompCtr=0;
   MQIPUTDataLenFindCd='N';
   MQPUTMsgOptsFindCd='N';
   putMsgOptsLine=0;
   MQPUTQueueStatsArCt=0;

   for (i = 0; i < QSTATMAX; i++)
      {
       for (j = 0; j < QNAMEMAX; j++)
           MQPUTQueueStats[i].QueueName[j]=' ';
       MQPUTQueueStats[i].break1='\0';
       MQPUTQueueStats[i].TotOk=0;
       MQPUTQueueStats[i].TotFailed=0;
       MQPUTQueueStats[i].TotDataLen=0;
       MQPUTQueueStats[i].TotOkElapsedTime=0;
       MQPUTQueueStats[i].TotFailedElapsedTime=0;
      }
   j=QSTATMAX-1;
   if (j >= 0)
      strncpy(MQPUTQueueStats[j].QueueName,"OVERFLOW SLOT: QUEUE NAME MAX COUNT EXCEEDED",44);

/* Initialize MQPUT1 variables */

   MQPUT1badFormat=0;
   MQPUT1BegFindCd='N';
   MQPUT1BegReasonFindCd='N';
   MQPUT1EndFindCd='Y';
   MQPUT1EndCompFindCd='N';
   MQPUT1EndCompCtr=0;
   MQIPUT1DataLenFindCd='N';
   MQPUT1MsgOptsFindCd='N';
   put1MsgOptsLine=0;
   put1BufferlengthLine=0;
   MQPUT1QueueStatsArCt=0;

   for (i = 0; i < QSTATMAX; i++)
      {
       for (j = 0; j < QNAMEMAX; j++)
           MQPUT1QueueStats[i].QueueName[j]=' ';
       MQPUT1QueueStats[i].break1='\0';
       MQPUT1QueueStats[i].TotOk=0;
       MQPUT1QueueStats[i].TotFailed=0;
       MQPUT1QueueStats[i].TotDataLen=0;
       MQPUT1QueueStats[i].TotOkElapsedTime=0;
       MQPUT1QueueStats[i].TotFailedElapsedTime=0;
      }
   j=QSTATMAX-1;
   if (j >= 0)
      strncpy(MQPUT1QueueStats[j].QueueName,"OVERFLOW SLOT: QUEUE NAME MAX COUNT EXCEEDED",44);

/* Begin loop to read formatted trace file */

   lineNmbr=0;

   while ( fgets(lineBuf, READMAX, fp) != NULL )
      {
       lineNmbr++;


/* MQOPEN logic */

       if (strstr(lineBuf,"MQOPEN >>"))
          {
           MQOPENBegFindCd='Y';
           if (MQOPENEndFindCd == 'N')
              {
               printf("MQOPEN begin found before MQOPEN end at line %d\n", lineNmbr);
               MQOPENbadFormat++;
              }
           MQOPENEndFindCd='N';
           MQOPENBegReasonFindCd='N'; 
           MQOPENEndCompFindCd='N';
           MQIOPENFindCd='N';
          }

       if (strstr(lineBuf,"Reason        :") && MQOPENBegFindCd == 'Y' && MQOPENEndFindCd == 'N')
          {
           MQOPENBegReasonFindCd='Y';
           strncpy(begTime,lineBuf+1,15);
          }
         
       if (strstr(lineBuf,"MQOPEN <<") && MQOPENBegFindCd == 'Y')
          {
           MQOPENEndFindCd='Y';
           strncpy(endTime,lineBuf+1,15);
          }

       if (MQOPENEndCompCtr == 1)
          {
           MQOPENEndCompCtr=0;
           if ((str1 = strstr(lineBuf,"0x0000:")) != 0)
              {
               MQOPENEndCompFindCd='Y';
               strncpy(Compcode,str1+9,8);
              }
          }

       if (strstr(lineBuf,"Compcode:") && MQOPENBegFindCd == 'Y' && MQOPENEndFindCd == 'Y')
          MQOPENEndCompCtr=1;

       if (strstr(lineBuf,"MQI:MQOPEN") != 0 && (str1 = strstr(lineBuf,"HConn=")) != 0 && (str2 = strstr(lineBuf,"ObjName=")) != 0) 
          {
           strncpy(HConn,str1+6,8);
           for (j = 0; j < QNAMEMAX; j++)
              QueueName[j]=' ';
           str3=str2;
           for ( ; *str3 != '\n' && *str3 != '\0'; str3++) ; 
           i = str3 - str2;
           if (i > 0 && i <= QNAMEMAX)
              strncpy(QueueName,str2+8,i); 
           MQIOPENFindCd='Y';

           if (MQOPENBegFindCd == 'Y' && MQOPENBegReasonFindCd == 'Y' && MQOPENEndFindCd == 'Y' && MQOPENEndCompFindCd == 'Y' && MQIOPENFindCd == 'Y')
              {
               elapsedTime = getElapsedTime(begTime, endTime);

               MQOPENQueueStatsFindCd='N';
               j=0;
               for (i=0; i < MQOPENQueueStatsArCt; i++)
                  {
                   if (strncmp(QueueName,MQOPENQueueStats[i].QueueName,QNAMEMAX) == 0 && 
                       strncmp(HConn,MQOPENQueueStats[i].HConn,HCONNMAX) == 0) 
                      {
                       MQOPENQueueStatsFindCd='Y';
                       j=i;
                      }
                  }
               if (MQOPENQueueStatsFindCd == 'N')
                  {
                   MQOPENQueueStatsArCt++;
                   if (MQOPENQueueStatsArCt == QSTATMAX)
                      {
                       MQOPENQueueStatsArCt--;
                       j=QSTATMAX-1;
                      }
                   else
                      {
                       j=MQOPENQueueStatsArCt-1;
                       strncpy(MQOPENQueueStats[j].QueueName,QueueName,QNAMEMAX);
                       strncpy(MQOPENQueueStats[j].HConn,HConn,HCONNMAX);
                      }
                  }
               if (atoi(Compcode) == 0)
                  {
                   MQOPENQueueStats[j].TotOk=MQOPENQueueStats[j].TotOk+1;
                   MQOPENQueueStats[j].TotOkElapsedTime=elapsedTime+MQOPENQueueStats[j].TotOkElapsedTime;
                  }
               else
                  {
                   MQOPENQueueStats[j].TotFailed=MQOPENQueueStats[j].TotFailed+1;
                   MQOPENQueueStats[j].TotFailedElapsedTime=elapsedTime+MQOPENQueueStats[j].TotFailedElapsedTime;
                  }
              }
           else
              {
               MQOPENbadFormat++;
               printf("MQOPEN begin or MQI missing when MQOPEN end was found at line %d\n", lineNmbr);
              }

           MQOPENBegFindCd='N';
           MQOPENBegReasonFindCd='N';
           MQOPENEndFindCd='Y';
           MQOPENEndCompFindCd='N';
           MQIOPENFindCd='N';
          }

/* MQCLOSE logic */

       if (strstr(lineBuf,"MQCLOSE >>"))
          {
           MQCLOSEBegFindCd='Y';
           if (MQCLOSEEndFindCd == 'N')
              {
               printf("MQCLOSE begin found before MQCLOSE end at line %d\n", lineNmbr);
               MQCLOSEbadFormat++;
              }
           MQCLOSEEndFindCd='N';
           MQICLOSEFindCd='N';
           MQCLOSEEndCompFindCd='N';
          }

       if (strstr(lineBuf,"MQI:MQCLOSE") != 0 && (str1 = strstr(lineBuf,"HConn=")) != 0)
          {
           strncpy(begTime,lineBuf+1,15);
           strncpy(HConn,str1+6,8);
           MQICLOSEFindCd='Y';
          }

       if (strstr(lineBuf,"MQCLOSE <<") && MQCLOSEBegFindCd == 'Y')
          {
           MQCLOSEEndFindCd='Y';
           strncpy(endTime,lineBuf+1,15);
          }

       if (MQCLOSEEndCompCtr == 1)
          {
           MQCLOSEEndCompCtr++;
           if ((str1 = strstr(lineBuf,"0x0000:")) != 0)
              {
               MQCLOSEEndCompFindCd='Y';
               strncpy(Compcode,str1+9,8);
              }
          }

       if (strstr(lineBuf,"Compcode:") && MQCLOSEBegFindCd == 'Y' && MQCLOSEEndFindCd == 'Y')
          MQCLOSEEndCompCtr=1;

       if (MQCLOSEEndCompCtr == 2)
          {
           MQCLOSEEndCompCtr=0;
           if (MQCLOSEBegFindCd == 'Y' && MQICLOSEFindCd == 'Y' && MQCLOSEEndFindCd == 'Y' && MQCLOSEEndCompFindCd == 'Y')
              {
               elapsedTime = getElapsedTime(begTime, endTime);

               MQCLOSEQueueStatsFindCd='N';
               j=0;
               for (i=0; i < MQCLOSEQueueStatsArCt; i++)
                  {
                   if (strncmp(HConn,MQCLOSEQueueStats[i].HConn,HCONNMAX) == 0)
                      {
                       MQCLOSEQueueStatsFindCd='Y';
                       j=i;
                      }
                  }
               if (MQCLOSEQueueStatsFindCd == 'N')
                  {
                   MQCLOSEQueueStatsArCt++;
                   if (MQCLOSEQueueStatsArCt == QSTATMAX)
                      {
                       MQCLOSEQueueStatsArCt--;
                       j=QSTATMAX-1;
                      }
                   else
                      {
                       j=MQCLOSEQueueStatsArCt-1;
                       strncpy(MQCLOSEQueueStats[j].HConn,HConn,HCONNMAX);
                      }
                  }
               if (atoi(Compcode) == 0)
                  {
                   MQCLOSEQueueStats[j].TotOk=MQCLOSEQueueStats[j].TotOk+1;
                   MQCLOSEQueueStats[j].TotOkElapsedTime=elapsedTime+MQCLOSEQueueStats[j].TotOkElapsedTime;
                  }
               else
                  {
                   MQCLOSEQueueStats[j].TotFailed=MQCLOSEQueueStats[j].TotFailed+1;
                   MQCLOSEQueueStats[j].TotFailedElapsedTime=elapsedTime+MQCLOSEQueueStats[j].TotFailedElapsedTime;
                  }
              }
           else
              {
               MQCLOSEbadFormat++;
               printf("MQCLOSE begin or MQI missing when MQCLOSE end was found at line %d\n", lineNmbr);
              }

           MQCLOSEBegFindCd='N';
           MQICLOSEFindCd='N';
           MQCLOSEEndFindCd='Y';
           MQCLOSEEndCompFindCd='N';
          }

/* MQGET logic */

       if (strstr(lineBuf,"MQGET >>"))
          {
           MQGETBegFindCd='Y';
           if (MQGETEndFindCd == 'N')
              {
               printf("MQGET begin found before MQGET end at line %d\n", lineNmbr);
               MQGETbadFormat++;
              }
           MQGETBegReasonFindCd='N';
           MQGETEndFindCd='N';
           MQGETMsgOptsFindCd='N';
           MQGETEndCompFindCd='N';
           MQIGETDataLenFindCd='N';
          }

       if (strstr(lineBuf,"Reason        :") && MQGETBegFindCd == 'Y' && MQGETEndFindCd == 'N')
          {
           MQGETBegReasonFindCd='Y';
           strncpy(begTime,lineBuf+1,15);
          }

       if (strstr(lineBuf,"MQGET <<") && MQGETBegFindCd == 'Y')
          {
           MQGETEndFindCd='Y';
           strncpy(endTime,lineBuf+1,15);
          }

       if (getMsgOptsLine > 0)
          {
           getMsgOptsLine++;
           if (getMsgOptsLine == 3 && (str1 = strstr(lineBuf,"0x0010")) != 0)
               strncpy(&QueueName[0],str1+55,8); 
           if (getMsgOptsLine == 4 && (str1 = strstr(lineBuf,"0x0020")) != 0)
               strncpy(&QueueName[8],str1+47,16);
           if (getMsgOptsLine == 5 && (str1 = strstr(lineBuf,"0x0030")) != 0)
               strncpy(&QueueName[24],str1+47,16);
           if (getMsgOptsLine == 6 && (str1 = strstr(lineBuf,"0x0040")) != 0)
               strncpy(&QueueName[40],str1+47,8);
           if (getMsgOptsLine > 6)
              getMsgOptsLine=0;
          }
 
       if (strstr(lineBuf,"Getmsgopts:") && MQGETEndFindCd == 'Y')
          {
           getMsgOptsLine=1;
           MQGETMsgOptsFindCd='Y'; 
           for (i = 0; i < QNAMEMAX; i++)
              QueueName[i]=' ';
          }
 
       if (MQGETEndCompCtr == 1)
          {
           MQGETEndCompCtr=0;
           if ((str1 = strstr(lineBuf,"0x0000:")) != 0)
              {
               MQGETEndCompFindCd='Y';
               strncpy(Compcode,str1+9,8);
              }
          }

       if (strstr(lineBuf,"Compcode:") && MQGETBegFindCd == 'Y' && MQGETEndFindCd == 'Y')
          MQGETEndCompCtr=1;

       if (strstr(lineBuf,"MQI:MQGET") != 0 && (str1 = strstr(lineBuf,"DataLen=")) != 0)
          {
           strncpy(HexStr,str1+8,8);
           MQIGETDataLenFindCd='Y';
           if (MQGETBegFindCd == 'Y' && MQGETBegReasonFindCd == 'Y' && MQGETEndFindCd == 'Y' && MQGETMsgOptsFindCd == 'Y' && MQGETEndCompFindCd == 'Y' && MQIGETDataLenFindCd == 'Y') 
              {
               elapsedTime = getElapsedTime(begTime, endTime);
               DataLen = ConvertHexToDec(HexStr,8);

               MQGETQueueStatsFindCd='N';
               j=0;
               for (i=0; i < MQGETQueueStatsArCt; i++)
                  {
                   if (strncmp(QueueName,MQGETQueueStats[i].QueueName,QNAMEMAX) == 0)
                      {
                       MQGETQueueStatsFindCd='Y';
                       j=i;
                      }
                  }
               if (MQGETQueueStatsFindCd == 'N')
                  {
                   MQGETQueueStatsArCt++;
                   if (MQGETQueueStatsArCt == QSTATMAX)
                      {
                       MQGETQueueStatsArCt--;
                       j=QSTATMAX-1;
                      }
                   else
                      {
                       j=MQGETQueueStatsArCt-1;
                       strncpy(MQGETQueueStats[j].QueueName,QueueName,QNAMEMAX);
                      }
                  } 
               MQGETQueueStats[j].TotDataLen=DataLen+MQGETQueueStats[j].TotDataLen;
               if (atoi(Compcode) == 0)
                  {
                   MQGETQueueStats[j].TotOk=MQGETQueueStats[j].TotOk+1;
                   MQGETQueueStats[j].TotOkElapsedTime=elapsedTime+MQGETQueueStats[j].TotOkElapsedTime;
                  }
               else
                  {
                   MQGETQueueStats[j].TotFailed=MQGETQueueStats[j].TotFailed+1;
                   MQGETQueueStats[j].TotFailedElapsedTime=elapsedTime+MQGETQueueStats[j].TotFailedElapsedTime;
                  }
              }
           else
              {
               MQGETbadFormat++;
               printf("MQGET begin or MQI or Getmsgopts missing when MQGET end was found at line %d\n", lineNmbr);
              }
           MQGETBegFindCd='N';
           MQGETBegReasonFindCd='N';
           MQIGETDataLenFindCd='N';
           MQGETEndFindCd='Y';
           MQGETEndCompFindCd='N';
           MQGETMsgOptsFindCd='N'; 
           getMsgOptsLine=0;
          }

/* MQPUT logic */

       if (strstr(lineBuf,"MQPUT >>"))
          {
           MQPUTBegFindCd='Y';
           if (MQPUTEndFindCd == 'N')
              {
               printf("MQPUT begin found before MQPUT end at line %d\n", lineNmbr);
               MQPUTbadFormat++;
              }
           MQPUTBegReasonFindCd='N';
           MQPUTEndFindCd='N';
           MQPUTMsgOptsFindCd='N';
           MQPUTEndCompFindCd='N';
           MQIPUTDataLenFindCd='N';
          }

       if (strstr(lineBuf,"Reason        :") && MQPUTBegFindCd == 'Y' && MQPUTEndFindCd == 'N')
          {
           MQPUTBegReasonFindCd='Y';
           strncpy(begTime,lineBuf+1,15);
          }

       if (strstr(lineBuf,"MQPUT <<") && MQPUTBegFindCd == 'Y')
          {
           MQPUTEndFindCd='Y';
           strncpy(endTime,lineBuf+1,15);
          }

       if (putMsgOptsLine > 0)
          {
           putMsgOptsLine++;
           if (putMsgOptsLine == 4 && (str1 = strstr(lineBuf,"0x0020")) != 0)
               strncpy(&QueueName[0],str1+47,16);
           if (putMsgOptsLine == 5 && (str1 = strstr(lineBuf,"0x0030")) != 0)
               strncpy(&QueueName[16],str1+47,16);
           if (putMsgOptsLine == 6 && (str1 = strstr(lineBuf,"0x0040")) != 0)
               strncpy(&QueueName[32],str1+47,16);
           if (putMsgOptsLine > 6)
              putMsgOptsLine=0;
          }

       if (strstr(lineBuf,"Putmsgopts:") && MQPUTEndFindCd == 'Y')
          {
           putMsgOptsLine=1;
           MQPUTMsgOptsFindCd='Y';
           for (i = 0; i < QNAMEMAX; i++)
              QueueName[i]=' ';
          }

       if (MQPUTEndCompCtr == 1)
          {
           MQPUTEndCompCtr=0;
           if ((str1 = strstr(lineBuf,"0x0000:")) != 0)
              {
               MQPUTEndCompFindCd='Y';
               strncpy(Compcode,str1+9,8);
              }
          }

       if (strstr(lineBuf,"Compcode:") && MQPUTBegFindCd == 'Y' && MQPUTEndFindCd == 'Y')
          MQPUTEndCompCtr=1;

       if (strstr(lineBuf,"MQI:MQPUT ") != 0 && (str1 = strstr(lineBuf,"BufLen=")) != 0)
          {
           strncpy(HexStr,str1+7,8);
           MQIPUTDataLenFindCd='Y';
           if (MQPUTBegFindCd == 'Y' && MQPUTBegReasonFindCd == 'Y' && MQPUTEndFindCd == 'Y' && MQPUTMsgOptsFindCd == 'Y' && MQPUTEndCompFindCd == 'Y' && MQIPUTDataLenFindCd == 'Y')
              {
               elapsedTime = getElapsedTime(begTime, endTime);
               DataLen = ConvertHexToDec(HexStr,8);

               MQPUTQueueStatsFindCd='N';
               j=0;
               for (i=0; i < MQPUTQueueStatsArCt; i++)
                  {
                   if (strncmp(QueueName,MQPUTQueueStats[i].QueueName,QNAMEMAX) == 0)
                      {
                       MQPUTQueueStatsFindCd='Y';
                       j=i;
                      }
                  }
               if (MQPUTQueueStatsFindCd == 'N')
                  {
                   MQPUTQueueStatsArCt++;
                   if (MQPUTQueueStatsArCt == QSTATMAX)
                      {
                       MQPUTQueueStatsArCt--;
                       j=QSTATMAX-1;
                      }
                   else
                      {
                       j=MQPUTQueueStatsArCt-1;
                       strncpy(MQPUTQueueStats[j].QueueName,QueueName,QNAMEMAX);
                      }
                  }
               MQPUTQueueStats[j].TotDataLen=DataLen+MQPUTQueueStats[j].TotDataLen;
               if (atoi(Compcode) == 0)
                  {
                   MQPUTQueueStats[j].TotOk=MQPUTQueueStats[j].TotOk+1;
                   MQPUTQueueStats[j].TotOkElapsedTime=elapsedTime+MQPUTQueueStats[j].TotOkElapsedTime;
                  }
               else
                  {
                   MQPUTQueueStats[j].TotFailed=MQPUTQueueStats[j].TotFailed+1;
                   MQPUTQueueStats[j].TotFailedElapsedTime=elapsedTime+MQPUTQueueStats[j].TotFailedElapsedTime;
                  }
              }
           else
              {
               MQPUTbadFormat++;
               printf("MQPUT begin or MQI or Putmsgopts missing when MQPUT end was found at line %d\n", lineNmbr);
              }

           MQPUTBegFindCd='N';
           MQPUTBegReasonFindCd='N';
           MQIPUTDataLenFindCd='N';
           MQPUTEndFindCd='Y';
           MQPUTEndCompFindCd='N';
           MQPUTMsgOptsFindCd='N';
           putMsgOptsLine=0;
          }

/* MQPUT1 logic */

       if (strstr(lineBuf,"MQPUT1 >>"))
          {
           MQPUT1BegFindCd='Y';
           if (MQPUT1EndFindCd == 'N')
              {
               printf("MQPUT1 begin found before MQPUT1 end at line %d\n", lineNmbr);
               MQPUT1badFormat++;
              }
           MQPUT1BegReasonFindCd='N';
           MQPUT1EndFindCd='N';
           MQPUT1MsgOptsFindCd='N';
           MQPUT1EndCompFindCd='N';
           MQIPUT1DataLenFindCd='N';
          }

       if (strstr(lineBuf,"Reason        :") && MQPUT1BegFindCd == 'Y' && MQPUT1EndFindCd == 'N')
          {
           MQPUT1BegReasonFindCd='Y';
           strncpy(begTime,lineBuf+1,15);
          }

       if (strstr(lineBuf,"MQPUT1 <<") && MQPUT1BegFindCd == 'Y')
          {
           MQPUT1EndFindCd='Y';
           strncpy(endTime,lineBuf+1,15);
          }

       if (put1MsgOptsLine > 0)
          {
           put1MsgOptsLine++;
           if (put1MsgOptsLine == 4 && (str1 = strstr(lineBuf,"0x0020")) != 0)
               strncpy(&QueueName[0],str1+47,16);
           if (put1MsgOptsLine == 5 && (str1 = strstr(lineBuf,"0x0030")) != 0)
               strncpy(&QueueName[16],str1+47,16);
           if (put1MsgOptsLine == 6 && (str1 = strstr(lineBuf,"0x0040")) != 0)
               strncpy(&QueueName[32],str1+47,16);
           if (put1MsgOptsLine > 6)
              put1MsgOptsLine=0;
          }

       if (strstr(lineBuf,"Putmsgopts:") && MQPUT1EndFindCd == 'Y')
          {
           put1MsgOptsLine=1;
           MQPUT1MsgOptsFindCd='Y';
           for (i = 0; i < QNAMEMAX; i++)
              QueueName[i]=' ';
          }

       if (put1BufferlengthLine > 0)
          {
           if ((str1 = strstr(lineBuf,"0x0000")) != 0)
              strncpy(HexStr,str1+9,8);
           put1BufferlengthLine=0;
          }

       if (strstr(lineBuf,"Bufferlength:") && MQPUT1BegFindCd == 'Y' && MQPUT1EndFindCd == 'N')
          {
           put1BufferlengthLine=1;
           MQIPUT1DataLenFindCd='Y'; 
           for (i = 0; i < HEXSTRMAX; i++)
              HexStr[i]='0';
          }

       if (MQPUT1EndCompCtr == 1)
          {
           MQPUT1EndCompCtr++;
           if ((str1 = strstr(lineBuf,"0x0000:")) != 0)
              {
               MQPUT1EndCompFindCd='Y';
               strncpy(Compcode,str1+9,8);
              }
          }

       if (strstr(lineBuf,"Compcode:") && MQPUT1BegFindCd == 'Y' && MQPUT1EndFindCd == 'Y')
          MQPUT1EndCompCtr=1;

       if (MQPUT1EndCompCtr == 2)
          {
           MQPUT1EndCompCtr=0;
           if (MQPUT1BegFindCd == 'Y' && MQPUT1BegReasonFindCd == 'Y' && MQPUT1EndFindCd == 'Y' && MQPUT1MsgOptsFindCd == 'Y' && MQPUT1EndCompFindCd == 'Y' && MQIPUT1DataLenFindCd == 'Y')
              {
               elapsedTime = getElapsedTime(begTime, endTime);
               if (littleEndian == 'Y')
                  DataLen = ConvertHexToDecLittleEndianLen8(HexStr);
               else
                  DataLen = ConvertHexToDec(HexStr,8); 

               MQPUT1QueueStatsFindCd='N';
               j=0;
               for (i=0; i < MQPUT1QueueStatsArCt; i++)
                  {
                   if (strncmp(QueueName,MQPUT1QueueStats[i].QueueName,QNAMEMAX) == 0)
                      {
                       MQPUT1QueueStatsFindCd='Y';
                       j=i;
                      }
                  }
               if (MQPUT1QueueStatsFindCd == 'N')
                  {
                   MQPUT1QueueStatsArCt++;
                   if (MQPUT1QueueStatsArCt == QSTATMAX)
                      {
                       MQPUT1QueueStatsArCt--;
                       j=QSTATMAX-1;
                      }
                   else
                      {
                       j=MQPUT1QueueStatsArCt-1;
                       strncpy(MQPUT1QueueStats[j].QueueName,QueueName,QNAMEMAX);
                      }
                  }
               MQPUT1QueueStats[j].TotDataLen=DataLen+MQPUT1QueueStats[j].TotDataLen;
               if (atoi(Compcode) == 0)
                  {
                   MQPUT1QueueStats[j].TotOk=MQPUT1QueueStats[j].TotOk+1;
                   MQPUT1QueueStats[j].TotOkElapsedTime=elapsedTime+MQPUT1QueueStats[j].TotOkElapsedTime;
                  }
               else
                  {
                   MQPUT1QueueStats[j].TotFailed=MQPUT1QueueStats[j].TotFailed+1;
                   MQPUT1QueueStats[j].TotFailedElapsedTime=elapsedTime+MQPUT1QueueStats[j].TotFailedElapsedTime;
                  }
              }
           else
              {
               MQPUT1badFormat++;
               printf("MQPUT1 begin or Bufferlength: or Putmsgopts missing when MQPUT1 end was found at line %d\n", lineNmbr);
              }
           MQPUT1BegFindCd='N';
           MQPUT1BegReasonFindCd='N';
           MQIPUT1DataLenFindCd='N';
           MQPUT1EndFindCd='Y';
           MQPUT1EndCompFindCd='N';
           MQPUT1MsgOptsFindCd='N';
           put1MsgOptsLine=0;
           put1BufferlengthLine=0; 
          }

      } /* While end bracket */

/* Write out number of successful reads for trace file */
   printf("\nNumber of successful lines read for trace file %s was %d\n",argv[1],lineNmbr);

/* Write out MQOPEN stats */

   printf("\n\nMQOPEN QUEUE STATS\n");

   printf("\nThe number of MQOPENs with bad formats was %d\n", MQOPENbadFormat);

   if (MQOPENQueueStatsArCt == 0)
      {
       printf("\nNo MQOPENs\n");
      }
   else
      {
       printf("\nFollowing rows have columns as follows:\n");
       printf("Queue Name, HConn, Total Ok OPEN, Total Ok OPEN Elapsed Time, Avg Ok OPEN Elapsed Time, Total Failed OPEN, Total Failed OPEN Elapsed Time, Avg Failed OPEN Elapsed Time\n\n");

       if ( (MQOPENQueueStatsArCt + 1) == QSTATMAX)
          MQOPENQueueStatsArCt++;

       for (i=0; i < MQOPENQueueStatsArCt; i++)
          {
           avgOkElapsedTime=0;
           avgFailedElapsedTime=0;
           if (MQOPENQueueStats[i].TotOk > 0)
              avgOkElapsedTime = MQOPENQueueStats[i].TotOkElapsedTime / MQOPENQueueStats[i].TotOk;
           if (MQOPENQueueStats[i].TotFailed > 0)
              avgFailedElapsedTime = MQOPENQueueStats[i].TotFailedElapsedTime / MQOPENQueueStats[i].TotFailed;
           printf("%s %s %d %012f %012f %d %012f %012f\n", MQOPENQueueStats[i].QueueName, MQOPENQueueStats[i].HConn, MQOPENQueueStats[i].TotOk, MQOPENQueueStats[i].TotOkElapsedTime, avgOkElapsedTime, MQOPENQueueStats[i].TotFailed, MQOPENQueueStats[i].TotFailedElapsedTime, avgFailedElapsedTime);  
          }
      }

/* Write out MQCLOSE stats */


   printf("\n\nMQCLOSE QUEUE STATS\n");

   printf("\nThe number of MQCLOSEs with bad formats was %d\n", MQCLOSEbadFormat);

   if (MQCLOSEQueueStatsArCt == 0)
      {
       printf("\nNo MQCLOSEs\n");
      }
   else
      {
       printf("\nFollowing rows have columns as follows:\n");
       printf("HConn, Total Ok CLOSE, Total Ok CLOSE Elapsed Time, Avg Ok CLOSE Elapsed Time, Total Failed CLOSE, Total Failed CLOSE Elapsed Time, Avg Failed CLOSE Elapsed Time\n\n");

       if ( (MQCLOSEQueueStatsArCt + 1) == QSTATMAX)
          MQCLOSEQueueStatsArCt++;

       for (i=0; i < MQCLOSEQueueStatsArCt; i++)
          {
           avgOkElapsedTime=0;
           avgFailedElapsedTime=0;
           if (MQCLOSEQueueStats[i].TotOk > 0)
              avgOkElapsedTime = MQCLOSEQueueStats[i].TotOkElapsedTime / MQCLOSEQueueStats[i].TotOk;
           if (MQCLOSEQueueStats[i].TotFailed > 0)
              avgFailedElapsedTime = MQCLOSEQueueStats[i].TotFailedElapsedTime / MQCLOSEQueueStats[i].TotFailed;
           printf("%s %d %012f %012f %d %012f %012f\n", MQCLOSEQueueStats[i].HConn, MQCLOSEQueueStats[i].TotOk, MQCLOSEQueueStats[i].TotOkElapsedTime, avgOkElapsedTime, MQCLOSEQueueStats[i].TotFailed, MQCLOSEQueueStats[i].TotFailedElapsedTime, avgFailedElapsedTime); 
          }
      }

/* Write out MQGET stats */

   printf("\n\nMQGET QUEUE STATS\n");

   printf("\nThe number of MQGETs with bad formats was %d\n", MQGETbadFormat);

   if (MQGETQueueStatsArCt == 0)
      {
       printf("\nNo MQGETs\n");
      } 
   else
      {
       printf("\nFollowing rows have columns as follows:\n");
       printf("Queue Name, Total Ok GET, Total Ok GET Elapsed Time, Avg Ok GET Elapsed Time, Total Failed GET, Total Failed GET Elapsed Time, Avg Failed GET Elapsed Time, Total GET Bytes\n\n"); 

       if ( (MQGETQueueStatsArCt + 1) == QSTATMAX) 
          MQGETQueueStatsArCt++;

       for (i=0; i < MQGETQueueStatsArCt; i++)
          {
           avgOkElapsedTime=0;
           avgFailedElapsedTime=0; 
           if (MQGETQueueStats[i].TotOk > 0)
              avgOkElapsedTime = MQGETQueueStats[i].TotOkElapsedTime / MQGETQueueStats[i].TotOk;
           if (MQGETQueueStats[i].TotFailed > 0)
              avgFailedElapsedTime = MQGETQueueStats[i].TotFailedElapsedTime / MQGETQueueStats[i].TotFailed;
           printf("%s %d %012f %012f %d %012f %012f %d\n", MQGETQueueStats[i].QueueName, MQGETQueueStats[i].TotOk, MQGETQueueStats[i].TotOkElapsedTime, avgOkElapsedTime, MQGETQueueStats[i].TotFailed, MQGETQueueStats[i].TotFailedElapsedTime, avgFailedElapsedTime, MQGETQueueStats[i].TotDataLen);
          }  
      }

/* Write out MQPUT stats */

   printf("\n\nMQPUT QUEUE STATS\n");

   printf("\nThe number of MQPUTs with bad formats was %d\n", MQPUTbadFormat);

   if (MQPUTQueueStatsArCt == 0)
      {
       printf("\nNo MQPUTs\n");
      } 
   else
      {
       printf("\nFollowing rows have columns as follows:\n");
       printf("Queue Name, Total Ok PUT, Total Ok PUT Elapsed Time, Avg Ok PUT Elapsed Time, Total Failed PUT, Total Failed PUT Elapsed Time, Avg Failed PUT Elapsed Time, Total PUT Bytes\n\n");

       if ( (MQPUTQueueStatsArCt + 1) == QSTATMAX)
          MQPUTQueueStatsArCt++;

       for (i=0; i < MQPUTQueueStatsArCt; i++)
          {
           avgOkElapsedTime=0;
           avgFailedElapsedTime=0;
           if (MQPUTQueueStats[i].TotOk > 0)
              avgOkElapsedTime = MQPUTQueueStats[i].TotOkElapsedTime / MQPUTQueueStats[i].TotOk;
           if (MQPUTQueueStats[i].TotFailed > 0)
              avgFailedElapsedTime = MQPUTQueueStats[i].TotFailedElapsedTime / MQPUTQueueStats[i].TotFailed;
           printf("%s %d %012f %012f %d %012f %012f %d\n", MQPUTQueueStats[i].QueueName, MQPUTQueueStats[i].TotOk, MQPUTQueueStats[i].TotOkElapsedTime, avgOkElapsedTime, MQPUTQueueStats[i].TotFailed, MQPUTQueueStats[i].TotFailedElapsedTime, avgFailedElapsedTime, MQPUTQueueStats[i].TotDataLen);
          }
      }

/* Write out MQPUT1 stats */

   printf("\n\nMQPUT1 QUEUE STATS\n");

   printf("\nThe number of MQPUT1s with bad formats was %d\n", MQPUT1badFormat);

   if (MQPUT1QueueStatsArCt == 0)
      {
       printf("\nNo MQPUT1s\n");
      }
   else
      {
       printf("\nFollowing rows have columns as follows:\n");
       printf("Queue Name, Total Ok PUT1, Total Ok PUT1 Elapsed Time, Avg Ok PUT1 Elapsed Time, Total Failed PUT1, Total Failed PUT1 Elapsed Time, Avg Failed PUT1 Elapsed Time, Total PUT1 Bytes\n\n"); 

       if ( (MQPUT1QueueStatsArCt + 1) == QSTATMAX)
          MQPUT1QueueStatsArCt++;

       for (i=0; i < MQPUT1QueueStatsArCt; i++)
          {
           avgOkElapsedTime=0;
           avgFailedElapsedTime=0;
           if (MQPUT1QueueStats[i].TotOk > 0)
              avgOkElapsedTime = MQPUT1QueueStats[i].TotOkElapsedTime / MQPUT1QueueStats[i].TotOk;
           if (MQPUT1QueueStats[i].TotFailed > 0)
              avgFailedElapsedTime = MQPUT1QueueStats[i].TotFailedElapsedTime / MQPUT1QueueStats[i].TotFailed;
           printf("%s %d %012f %012f %d %012f %012f %d\n", MQPUT1QueueStats[i].QueueName, MQPUT1QueueStats[i].TotOk, MQPUT1QueueStats[i].TotOkElapsedTime, avgOkElapsedTime, MQPUT1QueueStats[i].TotFailed, MQPUT1QueueStats[i].TotFailedElapsedTime, avgFailedElapsedTime, MQPUT1QueueStats[i].TotDataLen); 
          }
      }

   printf("\n");
   exit(0);
}

long ConvertHexToDec(char HexStr[], int len)
{
   int j;
   long base16, DecVl;

   base16=1;

   DecVl=0;

   len=len-1; 

   for (; len > -1; len--)
      {
       switch (HexStr[len])  {
       case '0': 
          j = 0;
          break;
       case '1': 
          j = 1;
          break;
       case '2': 
          j = 2;
          break;
       case '3': 
          j = 3;
          break;
       case '4':
          j = 4;
          break;
       case '5': 
          j = 5;
          break;
       case '6': 
          j = 6;
          break;
       case '7': 
          j = 7;
          break;
       case '8': 
          j = 8;
          break;
       case '9':
          j = 9;
          break;
       case 'A': 
          j = 10;
          break; 
       case 'B':
          j = 11;
          break; 
       case 'C':
          j = 12;
          break; 
       case 'D':
          j = 13;
          break; 
       case 'E':
          j = 14;
          break; 
       case 'F':
          j = 15;
          break; 
       }       

       DecVl = (j * base16) + DecVl;
       base16 = base16 * 16; 
      }

   return DecVl;
}

long ConvertHexToDecLittleEndianLen8(char HexStr[])
{
   int i, j, k;
   long base16, DecVl;

   base16=1;

   DecVl=0;

   for (i=0; i < 8; i++)
      {
       switch (i) {
       case 0:
          k=1;
          break;
       case 1:
          k=0;
          break;
       case 2:
          k=3;
          break;
       case 3:
          k=2;
          break;
       case 4:
          k=5;
          break;
       case 5:
          k=4;
          break;
       case 6:
          k=7;
          break;
       case 7:
          k=6;
          break;
       } 

       switch (HexStr[k])  {
       case '0':
          j = 0;
          break;
       case '1':
          j = 1;
          break;
       case '2':
          j = 2;
          break;
       case '3':
          j = 3;
          break;
       case '4':
          j = 4;
          break;
       case '5':
          j = 5;
          break;
       case '6':
          j = 6;
          break;
       case '7':
          j = 7;
          break;
       case '8':
          j = 8;
          break;
       case '9':
          j = 9;
          break;
       case 'A':
          j = 10;
          break;
       case 'B':
          j = 11;
          break;
       case 'C':
          j = 12;
          break;
       case 'D':
          j = 13;
          break;
       case 'E':
          j = 14;
          break;
       case 'F':
          j = 15;
          break;
       }

       DecVl = (j * base16) + DecVl;
       base16 = base16 * 16;
      }

   return DecVl;
}

double getElapsedTime(char begTime[], char endTime[])
{
   double hour, min, sec, subsec;
   double begTimeSeconds, endTimeSeconds, elapsedTime;

   hour = atoi(begTime);
   hour = hour * 3600;
   min  = atoi(begTime+3);
   min  = min * 60;
   sec  = atoi(begTime+6);
   subsec = atoi(begTime+9);
   subsec = subsec * 0.000001f;

   begTimeSeconds = hour + min + sec + subsec; 

   hour = atoi(endTime);
   hour = hour * 3600;
   min  = atoi(endTime+3);
   min  = min * 60;
   sec  = atoi(endTime+6);
   subsec = atoi(endTime+9);
   subsec = subsec * 0.000001f;

   endTimeSeconds = hour + min + sec + subsec; 

   elapsedTime = endTimeSeconds - begTimeSeconds;
   return elapsedTime;          
}
